from .jarmgr import *
from .jarmgr import _MY_DIR
from .pom import *
from .docker import docker_file
import platform
import os
import warnings
import os
from subprocess import call


def get_os():
    osname = platform.system()
    os_map = {
        'Windows': 'windows',
        'Linux': 'linux',
        'Darwin': 'mac'
    }
    if osname not in os_map:
        raise ValueError('{} platform is not supported.'.format(osname))
    return os_map[osname]


_CONFIG_FILE = os.path.join(_MY_DIR, 'config.json')



# Default config
_CONFIG = {
    'dl4j_version': '1.0.0-SNAPSHOT',
    'dl4j_core': True,
    'datavec': True,
    'spark': True,
    'spark_version': '2',
    'scala_version': '2.11',
    'nd4j_backend': 'cpu'
}


def _write_config():
    with open(_CONFIG_FILE, 'w') as f:
        json.dump(_CONFIG, f)    

if os.path.isfile(_CONFIG_FILE):
    with open(_CONFIG_FILE, 'r') as f:
        _CONFIG = json.load(f)
else:
    _write_config()


def set_config(config):
    _CONFIG.update(config)
    _write_config()


def get_config():
    return _CONFIG


def validate_config(config=None):
    if config is None:
        config = _CONFIG
    valid_options = {
        'spark_version': ['1', '2'],
        'scala_version': ['2.10', '2.11'],
        'nd4j_backend': ['cpu', 'gpu']
    }
    for k, vs in valid_options.items():
        v = config.get(k)
        if v is None:
            raise KeyError('Key not found in config : {}.'.format(k))
        if v not in vs:
            raise ValueError('Invalid value {} for key {} in config. Valid values are: {}.'.format(v, k, vs))

    # spark 2 does not work with scala 2.10
    if config['spark_version'] == '2' and config['scala_version'] == '2.10':
        raise ValueError('Scala 2.10 does not work with spark 2. Set scala_version to 2.11 in pydl4j config. ')


def _get_context_from_config():
    # e.g pydl4j-1.0.0-SNAPSHOT-cpu-spark2-2.11
    context = 'pydl4j-{}-{}-spark{}-{}'.format(
        _CONFIG['dl4j_version'],
        _CONFIG['nd4j_backend'],
        _CONFIG['spark_version'],
        _CONFIG['scala_version'])
    return context


set_context(_get_context_from_config())


def create_pom_from_config():
    config = get_config()
    pom = pom_template()
    dl4j_version = config['dl4j_version']
    nd4j_backend = config['nd4j_backend']
    use_spark = config['spark']
    scala_version = config['scala_version']
    spark_version = config['spark_version']
    use_dl4j_core = config['dl4j_core']
    use_datavec = config['datavec']

    datavec_deps = datavec_dependencies() if use_datavec else ""
    pom =pom.replace('{datavec.dependencies}', datavec_deps)

    core_deps = dl4j_core_dependencies() if use_dl4j_core else ""
    pom = pom.replace('{dl4j.core.dependencies}', core_deps)

    spark_deps = spark_dependencies() if use_spark else ""
    pom = pom.replace('{spark.dependencies}', spark_deps)

    pom = pom.replace('{dl4j.version}', dl4j_version)

    if nd4j_backend == 'cpu':
        platform_backend = "nd4j-native-platform"
        backend = "nd4j-native"
    else:
        platform_backend = "nd4j-cuda-9.2-platform"
        platform_backend = "nd4j-cuda-9.2"

    pom = pom.replace('{nd4j.backend}', backend)
    pom = pom.replace('{nd4j.platform.backend}', platform_backend)

    if use_spark:
        pom = pom.replace('{scala.binary.version}', scala_version)
        # this naming convention seems a little off
        if "SNAPSHOT" in dl4j_version:
            dl4j_version = dl4j_version.replace("-SNAPSHOT", "")
            dl4j_spark_version = dl4j_version + "_spark_" + spark_version + "-SNAPSHOT"
        else:
            dl4j_spark_version = dl4j_version + "_spark_" + spark_version
        pom = pom.replace('{dl4j.spark.version}', dl4j_spark_version)
    
    # TODO replace if exists
    pom_xml = os.path.join(_MY_DIR, 'pom.xml')
    with open(pom_xml, 'w') as pom_file:
        pom_file.write(pom)


def docker_build():
    docker_path = os.path.join(_MY_DIR, 'Dockerfile')
    docker_string = docker_file()
    with open(docker_path, 'w') as f:
        f.write(docker_string)

    call(["sudo", "docker", "build", _MY_DIR, "-t", "pydl4j"])


def docker_run():
    create_pom_from_config()
    call(["sudo", "docker", "run", "--mount", "src=" + _MY_DIR + ",target=/app,type=bind", "pydl4j"])
    # docker will build into <context>/target, need to move to context dir
    context_dir = get_dir()
    config = get_config()
    dl4j_version = config['dl4j_version']
    jar_name = "pydl4j-{}-bin.jar".format(dl4j_version)
    base_target_dir = os.path.join(_MY_DIR, "target")
    source = os.path.join(base_target_dir, jar_name)
    target = os.path.join(context_dir, jar_name)
    # os.rename or shutil won't work in all cases, need to assume sudo role
    call(["sudo", "mv", source, target])


def install_from_docker():
    docker_build()
    docker_run()


def install_docker_jars():
    jars = get_jars()
    dl4j_version = _CONFIG['dl4j_version']
    jar_name = "pydl4j-{}-bin.jar".format(dl4j_version)
    jar = os.path.join(get_dir(), jar_name)
    if not jar in jars:
       install_from_docker()


def _nd4j_jars():
    url = 'https://deeplearning4jblob.blob.core.windows.net/jars/'
    base_name = 'nd4j-uberjar'
    version = '1.0.0-SNAPSHOT'  # uploaded uber jar version. Version installed using docker can be different.
    jar_url = url + '{}-{}-{}-{}-no_avx.jar'.format(base_name, version, get_os(), _CONFIG['nd4j_backend'])
    jar_name = '{}-{}.jar'.format(base_name, version)
    return {base_name: [jar_url, jar_name]}


def _datavec_jars():
    url = 'https://deeplearning4jblob.blob.core.windows.net/jars/'
    base_name = 'datavec-uberjar'
    version = '1.0.0-SNAPSHOT'
    spark_v = _CONFIG['spark_version']
    scala_v = _CONFIG['scala_version']
    jar_url = url + base_name + '-{}-spark{}-{}.jar'.format(version, spark_v, scala_v)
    jar_name = '{}-{}.jar'.format(base_name, version)
    return {base_name: [jar_url, jar_name]}


def _validate_jars(jars):
    installed_jars = get_jars()
    for k, v in jars.items():
        found = False
        for j in installed_jars:
            if j.startswith(k):
                found = True
                break
        if not found:
            print('pydl4j: Required jar not installed {}.'.format(v[1]))
            config = get_config()
            # TODO: do we need other checks here?
            if config['nd4j_backend'] != 'gpu' or config['dl4j_version'] != '1.0.0-SNAPSHOT':
                install_docker_jars()
            else:
                install(v[0], v[1])


def _install_jars(jars):  # Note: downloads even if already installed.
    for v in jars.values():
        install(v[0], v[1])    


def install_nd4j_jars():
    _install_jars(_nd4j_jars)


def validate_nd4j_jars():
    _validate_jars(_nd4j_jars())


def install_datavec_jars():
    _install_jars(_datavec_jars())  


def validate_datavec_jars():
    _validate_jars(_datavec_jars())


def set_jnius_config():
    try:
        import jnius_config
        jnius_config.set_classpath(os.path.join(get_dir(), '*'))
    # Further options can be set by individual projects
    except ImportError:
        warnings.warn('Pyjnius not installed.')


def add_classpath(path):
    try:
        import jnius_config
        jnius_config.add_classpath(path)
    except ImportError:
        warnings.warn('Pyjnius not installed.') 


set_jnius_config()



from .downloader import download as download_file
from .mvn import *
import requests
import json
import os


def mkdir(x):
	if not os.path.isdir(x):
		os.mkdir(x)




_CONTEXT_NAME = None
_CONTEXT_DIR = None
_USER_PATH = os.path.expanduser('~')
_DL4J_DIR = os.path.join(_USER_PATH, '.deeplearning4j')
mkdir(_DL4J_DIR)
_MY_DIR = os.path.join(_DL4J_DIR, 'pydl4j')
mkdir(_MY_DIR)

_URLS_FILE = os.path.join(_MY_DIR, 'urls.json')
if os.path.isfile(_URLS_FILE):
    with open(_URLS_FILE, 'r') as f:
        _URLS = json.load(f)
else:
    _URLS = {}

def _write_urls():
    with open(_URLS_FILE, 'w') as f:
        json.dump(_URLS, f)

_cache = {}
def _read(url):
    text = _cache.get(url)
    if text is None:
        text = requests.get(url).text
        if not text:
            raise Exception('Empty response. Check connectivity.')
        _cache[url] = text
    return text


def _parse_contents(text):
    contents = text.split('<pre id="contents">')[1]
    contents = contents.split('</pre>')[0]
    contents = contents.split('<a href="')
    _ = contents.pop(0)
    link_to_parent = contents.pop(0)
    contents = list(map(lambda x: x.split('"')[0], contents))
    contents = [c[:-1] for c in contents if c[-1] == '/']  # removes meta data files
    return contents


def check(f):
    def wrapper(*args, **kwargs):
        if _CONTEXT_NAME is None:
            raise Exception('Context not set! Set context using pydl4j.set_context()')
        mkdir(_CONTEXT_DIR)
        return f(*args, **kwargs)
    return wrapper    


def set_context(name):
    global _CONTEXT_NAME
    global _CONTEXT_DIR
    _CONTEXT_NAME = name
    if name is None:
        _CONTEXT_DIR = None
    else:
        _CONTEXT_DIR = os.path.join(_MY_DIR, name)
        mkdir(_CONTEXT_DIR)


@check
def context():
    return _CONTEXT_NAME


@check
def get_dir():
    return _CONTEXT_DIR


@check
def install(url, jar_name=None):
    if not jar_name:
        jar_name = os.path.basename(url)
    jar_path = os.path.join(_CONTEXT_DIR, jar_name)
    temp_jar_path = jar_path + '.tmp'
    if os.path.isfile(temp_jar_path):
        os.remove(temp_jar_path)
    download_file(url, temp_jar_path)
    if os.path.isfile(jar_path):
        os.remove(jar_path)
    os.rename(temp_jar_path, jar_path)
    if _CONTEXT_NAME not in _URLS:
        _URLS[_CONTEXT_NAME] = {jar_name: url}
    else:
        _URLS[_CONTEXT_NAME][jar_name] = url
    _write_urls()


@check
def mvn_install(group, artifact, version=None):
    if version is None:
        version = get_latest_version(group, artifact)
        print('Version not specified for org.{}.{}.'
              'Installing latest version: {}.'.format(group, artifact, version))
    url = get_jar_url(group, artifact, version)
    install(url)

@check
def uninstall(artifact, version=None):
    files = os.listdir(_CONTEXT_DIR)
    if version is not None:
        artifact += '-' + version
    if not artifact.endswith('.jar'):
        artifact += '.jar'
    found = False
    for f in files:
        if f == artifact:  # could have wildcards, disabled for now
            os.remove(os.path.join(_CONTEXT_DIR, f))
            if _CONTEXT_NAME in _URLS:
                if f in _URLS[_CONTEXT_NAME]:
                    _URLS[_CONTEXT_NAME].pop(f)
                    _write_urls()
            found = True
            break
    if not found:
        raise Exception('No matching jars found : {}. '
                        'Use pydl4j.get_jars() to see available jars.'.format(artifact))


@check
def get_jars():
    return [x for x in os.listdir(_CONTEXT_DIR) if x.endswith('.jar')]


@check
def clear_context():
    for j in get_jars():
        uninstall(j)
    try:
        os.remove(_CONTEXT_DIR)
    except:
        pass


def update(jar):
    install(_URLS[_CONTEXT_NAME][jar])



from .progressbar import ProgressBar
import requests
import math
import os
import hashlib


def download(url, file_name):
    #u = urlopen(url)
    r = requests.get(url, stream=True)
    file_size = int(r.headers['Content-length'])
    '''
    if py3:
        file_size = int(u.getheader("Content-Length")[0])
    else:
        file_size = int(u.info().getheaders("Content-Length")[0])
    '''
    file_exists = False
    if os.path.isfile(file_name):
        local_file_size = os.path.getsize(file_name)
        if local_file_size == file_size:
            sha1_file = file_name + '.sha1'
            if os.path.isfile(sha1_file):
                print('sha1 found')
                with open(sha1_file) as f:
                    expected_sha1 = f.read()
                BLOCKSIZE = 65536
                sha1 = hashlib.sha1()
                with open(file_name) as f:
                    buff = f.read(BLOCKSIZE)
                    while len(buff) > 0:
                        sha1.update(buff)
                        buff = f.read(BLOCKSIZE)
                if expected_sha1 == sha1:
                    file_exists = True
                else:
                    print("File corrupt. Downloading again.")
                    os.remove(file_name)
            else:
                file_exists = True
        else:
            print("File corrupt. Downloading again.")
            os.remove(file_name)
    if not file_exists:
        factor = int(math.floor(math.log(file_size)/math.log(1024)))
        display_file_size = str(file_size / 1024 ** factor) + ['B','KB','MB','GB','TB','PB'][factor]
        print("Source: " + url)
        print("Destination " + file_name)
        print("Size: " + display_file_size)
        file_size_dl = 0
        block_sz = 8192
        f = open(file_name, 'wb')
        pbar = ProgressBar(file_size)
        for chunk in r.iter_content(chunk_size=block_sz):
            if not chunk:
                continue
            chunk_size = len(chunk)
            file_size_dl += chunk_size
            f.write(chunk)
            pbar.update(chunk_size)
            #status = r"%10d  [%3.2f%%]" % (file_size_dl, file_size_dl * 100. / file_size)
            #status = status + chr(8)*(len(status)+1)
            #print(status)
        f.close()
    else:
        print("File already exists - " + file_name)
        return True



#!/usr/bin python
# -*- coding: utf-8 -*-

import argparse
import json
import os
import sys
import pkg_resources
import argcomplete
import traceback
import subprocess
from builtins import input

import click
from click.exceptions import ClickException
from dateutil import parser

from .pydl4j import set_config, get_config, install_from_docker
from .pydl4j import validate_config
# from .pydl4j import install_docker_jars

_CONFIG = get_config()

DEFAULT_DL4J_VERSION = '1.0.0-beta2' # doesn't currently work with snapshots :/
DEFAULT_BACKEND = _CONFIG['nd4j_backend']
DEFAULT_DATAVEC = _CONFIG['datavec']
DEFAULT_SPARK = _CONFIG['spark']
DEFAULT_SPARK_MAJOR = _CONFIG['spark_version']
DEFAULT_SCALA_VERSION = _CONFIG['scala_version']
DEFAULT_SPARK_DETAILS = 'y'


def to_bool(string):
    if type(string) is bool:
        return string
    return True if string[0] in ["Y", "y"] else False

class CLI(object):
    
    def __init__(self):
        self.var_args = None
        self.command = None

    def command_dispatcher(self, args=None):
        desc = ('pydl4j,  a system to manage your DL4J dependencies from Python.\n')
        parser = argparse.ArgumentParser(description=desc)
        parser.add_argument(
            '-v', '--version', action='version',
            version=pkg_resources.get_distribution("pydl4j").version,
            help='Print pydl4j version'
        )

        subparsers = parser.add_subparsers(title='subcommands', dest='command')
        subparsers.add_parser('init', help='Initialize pydl4j')
        subparsers.add_parser('install', help='Install jars for pydl4j')


        argcomplete.autocomplete(parser)
        args = parser.parse_args(args)
        self.var_args = vars(args)

        if not args.command:
            parser.print_help()
            return

        self.command = args.command

        if self.command == 'init':
            self.init()
            return
        
        if self.command == 'install':
            self.install()
            return



    def init(self):

        click.echo(click.style(u"""\n██████╗ ██╗   ██╗██████╗ ██╗██╗  ██╗     ██╗
██╔══██╗╚██╗ ██╔╝██╔══██╗██║██║  ██║     ██║
██████╔╝ ╚████╔╝ ██║  ██║██║███████║     ██║
██╔═══╝   ╚██╔╝  ██║  ██║██║╚════██║██   ██║
██║        ██║   ██████╔╝███████╗██║╚█████╔╝
╚═╝        ╚═╝   ╚═════╝ ╚══════╝╚═╝ ╚════╝ \n""", fg='blue', bold=True))

        click.echo(click.style("pydl4j", bold=True) + " is a system to manage your DL4J dependencies from Python!\n")

        # DL4J version
        dl4j_version = input("Which DL4J version do you want to use for your Python projects? (default '%s'): " % DEFAULT_DL4J_VERSION) or DEFAULT_DL4J_VERSION
        # TODO: check if input is valid

        # ND4J backend
        backend = input("Which backend would you like to use ('cpu' or 'gpu')? (default '%s'): " % DEFAULT_BACKEND) or DEFAULT_BACKEND
        backend = backend.lower()

        # DataVec usage
        datavec = input("Do you need DL4J DataVec for ETL? (default 'y') [y/n]: ") or DEFAULT_DATAVEC
        datavec = to_bool(datavec)

        # DL4J core usage
        DEFAULT_DL4J = 'y'
        dl4j_core = input("Do you want to work with DeepLearning4J from Python? (default 'y') [y/n]: ") or DEFAULT_DL4J
        dl4j_core = to_bool(dl4j_core)

        # Spark
        spark = input("Do you need Spark for distributed computation in your application? (default 'y') [y/n]: ") or DEFAULT_SPARK
        spark = to_bool(spark)
        spark_version = DEFAULT_SPARK_MAJOR
        scala_version = DEFAULT_SCALA_VERSION
        if spark:
            spark_details = input("We use Spark {} and Scala {} by default, is this OK for you? (default 'y') [y/n]: ".format(DEFAULT_SPARK_MAJOR,
                DEFAULT_SCALA_VERSION)) or DEFAULT_SPARK_DETAILS
            if not spark_details[0] in ["Y", "y"]:
                spark_version = input("Which which major Spark release would you like to use? (default '%s'): " % DEFAULT_SPARK_MAJOR) or DEFAULT_SPARK_MAJOR
                scala_version = input("Which Scala version would you like to use? (default '%s'): " % DEFAULT_SCALA_VERSION) or DEFAULT_SCALA_VERSION


        cli_out = {
                'dl4j_version': dl4j_version,
                'nd4j_backend': backend,
                'dl4j_core': dl4j_core,
                'datavec': datavec,
                'spark': spark,
                'spark_version': spark_version,
                'scala_version': scala_version
        }

        validate_config(cli_out)
        formatted_json = json.dumps(cli_out, sort_keys=False, indent=2)

        click.echo("\nThis is your current settings file " + click.style("config.json", bold=True) + ":\n")
        click.echo(click.style(formatted_json, fg="green", bold=True))

        confirm = input("\nDoes this look good? (default 'y') [y/n]: ") or 'yes'
        if not to_bool(confirm):
            click.echo("" + click.style("Please initialize pydl4j once again", fg="red", bold=True))
            return

        set_config(cli_out)

    def install(self):
        # TODO: optionally download uberjars
        check_docker()

        click.echo(click.style("========\n\nNote that this might take some time to complete.\n" + \
        "We will first pull a docker container with Maven, then install all dependencies selected with 'pydl4j init'.\n" + \
        "After completion you can start using DL4J from Python.\n\n========", fg="green", bold=False))

        install_from_docker()


def handle():
    try:
        cli = CLI()
        sys.exit(cli.command_dispatcher())
    except KeyboardInterrupt:
        sys.exit()
    except Exception as e:
        click.echo(click.style("Error: ", fg='red', bold=True))
        traceback.print_exc()
        sys.exit()


def check_docker():
    devnull = open(os.devnull, 'w')
    try:
        subprocess.call(["docker", "--help"], stdout=devnull, stderr=devnull)
        click.echo(click.style("Docker is running, starting installation.", fg="green", bold=True))
    except:
        click.echo("" + click.style("Could not detect docker on your system. Make sure a docker deamon is running", fg="red", bold=True))
        raise Exception("Aborting installation, docker not found.")

if __name__ == '__main__':
    handle()



def dl4j_core_dependencies():
    return """<dependency>
            <groupId>org.deeplearning4j</groupId>
            <artifactId>deeplearning4j-core</artifactId>
            <version>${project.version}</version>
        </dependency>"""


def spark_dependencies():
    return """<dependency>
            <groupId>org.deeplearning4j</groupId>
            <artifactId>dl4j-spark-parameterserver_{scala.binary.version}</artifactId>
            <version>{dl4j.spark.version}</version>
        </dependency>"""

def datavec_dependencies():
    return """<dependency>
            <groupId>org.datavec</groupId>
            <artifactId>datavec-api</artifactId>
            <version>${project.version}</version>
        </dependency>
        <dependency>
            <groupId>org.datavec</groupId>
            <artifactId>datavec-arrow</artifactId>
            <version>${project.version}</version>
        </dependency>
        <dependency>
            <groupId>org.datavec</groupId>
            <artifactId>datavec-camel</artifactId>
            <version>${project.version}</version>
        </dependency>
        <dependency>
            <groupId>org.datavec</groupId>
            <artifactId>datavec-excel</artifactId>
            <version>${project.version}</version>
        </dependency>
        <dependency>
            <groupId>org.datavec</groupId>
            <artifactId>datavec-geo</artifactId>
            <version>${project.version}</version>
        </dependency>
        <dependency>
            <groupId>org.datavec</groupId>
            <artifactId>datavec-hadoop</artifactId>
            <version>${project.version}</version>
        </dependency>
        <dependency>
            <groupId>org.datavec</groupId>
            <artifactId>datavec-jdbc</artifactId>
            <version>${project.version}</version>
        </dependency>
        <dependency>
            <groupId>org.datavec</groupId>
            <artifactId>datavec-perf</artifactId>
            <version>${project.version}</version>
        </dependency> 
        <dependency>
            <groupId>org.datavec</groupId>
            <artifactId>datavec-spark_{scala.binary.version}</artifactId>
            <version>{dl4j.spark.version}</version>
        </dependency>
        <!-- <dependency>
            <groupId>org.datavec</groupId>
            <artifactId>datavec-spark-inference-parent</artifactId>
            <version>${project.version}</version>
        </dependency> -->
"""
    


def pom_template():
    return """<?xml version="1.0" encoding="UTF-8"?>
<!--~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  ~ Copyright (c) 2015-2018 Skymind, Inc.
  ~
  ~ This program and the accompanying materials are made available under the
  ~ terms of the Apache License, Version 2.0 which is available at
  ~ https://www.apache.org/licenses/LICENSE-2.0.
  ~
  ~ Unless required by applicable law or agreed to in writing, software
  ~ distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
  ~ WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
  ~ License for the specific language governing permissions and limitations
  ~ under the License.
  ~
  ~ SPDX-License-Identifier: Apache-2.0
  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~-->

<project xmlns="http://maven.apache.org/POM/4.0.0"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xsi:schemaLocation="http://maven.apache.org/POM/4.0.0 http://maven.apache.org/maven-v4_0_0.xsd">


    <modelVersion>4.0.0</modelVersion>
    <groupId>org.deeplearning4j</groupId>
    <artifactId>pydl4j</artifactId>
    <version>{dl4j.version}</version>
    <packaging>jar</packaging>

    <name>pydl4j</name>

    <properties>
        <maven-shade-plugin.version>3.0.0</maven-shade-plugin.version>
        <javacpp.version>1.4.3-SNAPSHOT</javacpp.version>
        <javacpp-presets.version>1.4.2</javacpp-presets.version>
        <openblas.version>0.3.0</openblas.version>
    </properties>

    <licenses>
        <license>
            <name>Apache License, Version 2.0</name>
            <url>http://www.apache.org/licenses/LICENSE-2.0.txt</url>
            <distribution>repo</distribution>
        </license>
    </licenses>

    <dependencies>
        <dependency>
            <groupId>org.bytedeco.javacpp-presets</groupId>
            <artifactId>openblas-platform</artifactId>
            <version>${openblas.version}-${javacpp-presets.version}</version>
        </dependency>
        <dependency>
            <groupId>org.nd4j</groupId>
            <artifactId>{nd4j.platform.backend}</artifactId>
            <version>${project.version}</version>
        </dependency>
        <dependency>
            <groupId>org.nd4j</groupId>
            <artifactId>{nd4j.backend}</artifactId>
            <version>${project.version}</version>
            <classifier>linux-x86_64</classifier>
        </dependency>
        <dependency>
            <groupId>org.nd4j</groupId>
            <artifactId>{nd4j.backend}</artifactId>
            <version>${project.version}</version>
            <classifier>windows-x86_64</classifier>
        </dependency>
        {dl4j.core.dependencies}
        {spark.dependencies}
        {datavec.dependencies}
        <dependency>
            <groupId>ch.qos.logback</groupId>
            <artifactId>logback-classic</artifactId>
            <version>1.2.3</version>
        </dependency>
    </dependencies>

    <repositories>
        <repository>
            <id>snapshots-repo</id>
            <url>https://oss.sonatype.org/content/repositories/snapshots</url>
            <releases>
                <enabled>false</enabled>
            </releases>
            <snapshots>
                <enabled>true</enabled>
                <updatePolicy>daily</updatePolicy>  <!-- Optional, update daily -->
            </snapshots>
        </repository>
    </repositories>

    <build>
        <plugins>
            <plugin>
                <groupId>org.apache.maven.plugins</groupId>
                <artifactId>maven-shade-plugin</artifactId>
                <version>${maven-shade-plugin.version}</version>
                <configuration>
                    <shadedArtifactAttached>true</shadedArtifactAttached>
                    <shadedClassifierName>bin</shadedClassifierName>
                    <createDependencyReducedPom>true</createDependencyReducedPom>
                    <filters>
                        <filter>
                            <artifact>*:*</artifact>
                            <excludes>
                                <exclude>org/datanucleus/**</exclude>
                                <exclude>META-INF/*.SF</exclude>
                                <exclude>META-INF/*.DSA</exclude>
                                <exclude>META-INF/*.RSA</exclude>
                            </excludes>
                        </filter>
                    </filters>
                </configuration>
                <executions>
                    <execution>
                        <phase>package</phase>
                        <goals>
                            <goal>shade</goal>
                        </goals>
                        <configuration>
                            <transformers>
                                <transformer implementation="org.apache.maven.plugins.shade.resource.AppendingTransformer">
                                    <resource>reference.conf</resource>
                                </transformer>
                                <transformer implementation="org.apache.maven.plugins.shade.resource.ServicesResourceTransformer"/>
                                <transformer implementation="org.apache.maven.plugins.shade.resource.ManifestResourceTransformer">
                                </transformer>
                            </transformers>
                        </configuration>
                    </execution>
                </executions>
            </plugin>
            <plugin>
                <groupId>org.apache.maven.plugins</groupId>
                <artifactId>maven-compiler-plugin</artifactId>
                <version>3.1</version>
                <configuration>
                    <source>1.8</source>
                    <target>1.8</target>
                </configuration>
            </plugin>
            <plugin>
                <groupId>org.apache.maven.plugins</groupId>
                <artifactId>maven-jar-plugin</artifactId>
                <configuration>
                    <forceCreation>true</forceCreation>
                </configuration>
                <executions>
                    <execution>
                        <id>empty-javadoc-jar</id>
                        <phase>package</phase>
                        <goals>
                            <goal>jar</goal>
                        </goals>
                        <configuration>
                            <skip>false</skip>
                            <classifier>javadoc</classifier>
                            <classesDirectory>${basedir}/javadoc</classesDirectory>
                        </configuration>
                    </execution>
                    <execution>
                        <id>empty-sources-jar</id>
                        <phase>package</phase>
                        <goals>
                            <goal>jar</goal>
                        </goals>
                        <configuration>
                            <classifier>sources</classifier>
                            <classesDirectory>${basedir}/src</classesDirectory>
                        </configuration>
                    </execution>
                </executions>
            </plugin>
        </plugins>
    </build>
</project>


    """


from setuptools import setup
from setuptools import find_packages

setup(
    name='pydl4j',
    version=0.1,
    packages=find_packages(),
    install_requires=[],
    include_package_data=True,
    license='MIT',
    description='Java dependency management for Python projects using DL4J',
    url='https://github.com/deeplearning4j/pydl4j',
    entry_points={
        'console_scripts': [
            'pydl4j=pydl4j.cli:handle'
        ]
    },
    classifiers=[
        'Environment :: Console',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3'
    ]
)



from .downloader import download as download_file
import requests


_cache = {}
def _read(url):
    text = _cache.get(url)
    if text is None:
        text = requests.get(url).text
        if not text:
            raise Exception('Empty response. Check connectivity.')
        _cache[url] = text
    return text


def _parse_contents(text):
    contents = text.split('<pre id="contents">')[1]
    contents = contents.split('</pre>')[0]
    contents = contents.split('<a href="')
    _ = contents.pop(0)
    link_to_parent = contents.pop(0)
    contents = list(map(lambda x: x.split('"')[0], contents))
    contents = [c[:-1] for c in contents if c[-1] == '/']  # removes meta data files
    return contents



def get_artifacts(group):
    url = ('https://search.maven.org/remotecontent?filepath=' +
          'org/{}/'.format(group))
    response = _read(url)
    return _parse_contents(response)


def get_versions(group, artifact):
    url = ('https://search.maven.org/remotecontent?filepath=' +
          'org/{}/{}/'.format(group, artifact))
    response = _read(url)
    return _parse_contents(response)


def get_latest_version(group, artifact):
    return get_versions(group, artifact)[-1]


def get_jar_url(group, artifact, version=None):
    if version is None:
        version = get_versions(group, artifact)[-1]
    url = ('http://search.maven.org/remotecontent?filepath=' + 
          'org/{}/{}/{}/{}-{}.jar'.format(group, artifact, version,
                                          artifact, version))
    return url



def docker_file():
    return """FROM java:openjdk-8-jdk
ENV MAVEN_VERSION 3.3.9

RUN curl -fsSL http://archive.apache.org/dist/maven/maven-3/$MAVEN_VERSION/binaries/apache-maven-$MAVEN_VERSION-bin.tar.gz | tar xzf - -C /usr/share \
  && mv /usr/share/apache-maven-$MAVEN_VERSION /usr/share/maven \
  && ln -s /usr/share/maven/bin/mvn /usr/bin/mvn

ENV MAVEN_HOME /usr/share/maven

# Copy application to container
RUN mkdir -p app
WORKDIR /app

CMD ["mvn", "package"]
"""
